/***********************************************
*                                              *
*      action_planner.h                        *
*                                              *
*      Jesus Savage                            *
*      Julio Cruz                              *
*                                              *
*              Bio-Robotics Laboratory         *
*              UNAM, 2020                      *
*                                              *
*                                              *
************************************************/


#include "ros/ros.h"
#include "simulator/simulator_find_obj.h"
#include "simulator/simulator_manipulator.h"
#include <iostream>
#include <vector>
#include <ctime>
#include <map>
#include <sstream>
#include <string>
#include "simulator/MotionPlanner.h"
//#include "simulator/ActionPlanner.h"

#ifndef PI
#define PI 3.1415926535f
#endif

int flg_msg = 0;

// Message callback function. This is a function is called when a topic
// message named 'motion_planner_msg' is received. As an input message,
// the 'MsgTutorial' message of the 'ros_tutorials_topic' package is received.
//void msgCallbackMotionPlanner(const simulator::MotionPlanner::ConstPtr& msg)
void msgCallbackMotionPlanner(const simulator::MotionPlanner::ConstPtr& msg)
{

        ROS_INFO("received msg from motion planner = %d", msg->stamp.sec);   // Shows the 'stamp.sec' message
        ROS_INFO("received msg from motion planner = %d", msg->stamp.nsec);  // Shows the 'stamp.nsec' message
        ROS_INFO("received msg from motion planner = %d", msg->data);        // Shows the 'data' message
        //printf("received msg from motion  planner %s\n",msg->motion.c_str());     // Shows the 'motion' message
	ROS_INFO("recieve msg motion planner = %s", msg->motion.c_str());       // Shows the 'motion' message

        action_data = msg->data;
        //strcpy(Motion_Planner,msg->motion.c_str());
	flg_msg = 1;
}


// It starts the communication with the CLIPS node
int start_clips_node_action_planner(){

 bool init_kdb = false;
 std::string file;
 std::string result;

 std::cout << "Starting CLIPS" << std::endl;


 //This functions loads initial facts and rules from a file
 // The first parameter is a file that contains the names of CLIPS files *.clp
 // The second parameter indicates with false do not start executing the loaded files
 // The third parameter is a timeout
 //file = "/src/expert_system/oracle.dat";
 file = "/src/action_planner/ViRBot_Cubes_ROS/ROS_cubes.dat";
 std::cout << "CLIPS file: " << file  << std::endl;
 init_kdb = SimuladorRepresentation::initKDB(file, false, 2000);
 if(!init_kdb){
                std::cout << "CLIPS error file not found: " << file  << std::endl;
                return 0;
 }

 //Function to RESET CLIPS
 SimuladorRepresentation::resetCLIPS(true);

 //Function to print facts 
 SimuladorRepresentation::factCLIPS(true);

 //Function to print the loaded rules' names
 SimuladorRepresentation::ruleCLIPS(true);

 //Function to start running Clips
 SimuladorRepresentation::runCLIPS(true);

 //Function to asserting a fact to the clips node to check if Clips is alive
 SimuladorRepresentation::strQueryKDB("(assert (alive clips))", result, 10000);

 std::cout << "CLIPS answer: " << result << std::endl;

}




void send_grab_drop(char *action, char *object, char *answer){

    printf("%s object %s\n",action,object);
    char str[300];
    char ROS_System[30];

    ros::NodeHandle n;
    ros::ServiceClient client;
    simulator::simulator_manipulator srv;
    client = n.serviceClient<simulator::simulator_manipulator>("simulator_manipulator"); //create the client

    srv.request.ROS_System = ROS_System;
    srv.request.action = action;
    srv.request.object = object;

    if (client.call(srv))
    {
      strcpy(str,srv.response.answer.c_str());
    }
    else
    {
      ROS_ERROR("Failed to call service simulator_manipulator");
    }

    //printf("\nAnswer manipulator %s\n",str);
    strcpy(answer,str);
    //SimuladorRepresentation::strQueryKDB(str, result, 10000);
    //printf("\nCLIPS answer: %d %s\n",i,result.c_str());

}




int action_planner(float px, float py, float theta, Actions *plan){

       static int mini_sm=1;
       static char object_name[200];
       static int init_flg=1;
       std::string result;
       static int i=0,j=0,k=0;
       char str[300];
       char action[30];
       char ROS_System[30];
       char room[30];
       char zone[30];
       static float x,y,z=0.0;
       static char arm[30];
       static char object[100];
       static float distance=1.0;
       static float angle=0.0;
       static int num_plans;
       //static char plan[100][300];


       //movements->twist = 0.0;
       //movements->advance = 0.0;

       if(init_flg==1){

       		// It starts the communication with the Clips node
       		start_clips_node_action_planner();
       		init_flg=0;
       		//strcpy(arm,"manipulator");

        	SimuladorRepresentation::strQueryKDB("(assert (start action-planning))", result, 10000);
          	std::cout << "CLIPS answer: " << result << std::endl;
       }

       i++;
       sprintf(str,"(assert (step %d ))",i);
       printf("\nSend fact %s\n",str);
       SimuladorRepresentation::strQueryKDB(str, result, 10000);
       printf("\nCLIPS answer: %d %s\n",i,result.c_str());

       sscanf(result.c_str(),"%s %s",ROS_System,action);
       printf("ROS_System %s action %s\n",ROS_System,action);

       if(strcmp(action,"num_plans")==0){
		sscanf(result.c_str(),"%s %s %d",ROS_System,action,&num_plans);
		printf("num_plans %d\n",num_plans);
		for(k=0; k< num_plans;k++){
       			sprintf(str,"(assert (step %d ))",num_plans-k);
       			printf("\nSend fact %s\n",str);
       			SimuladorRepresentation::strQueryKDB(str, result, 10000);
       			printf("\nPLAN received: %d %s\n",num_plans-k,result.c_str());
			strcpy(plan->action_plan[num_plans-k],result.c_str());
		}

		for(k=1; k<= num_plans;k++){
			printf("PLAN to be executed: %d %s\n",k,plan->action_plan[k]);
		}

       }


 return num_plans;

}


// It notifies to the action planner in CLIPS that action plan number was executed
void send_clips(int num){

 char str[300];
 std::string result;

 sprintf(str,"(assert (executed %d ))",num);
 printf("\nSend fact %s\n",str);
 SimuladorRepresentation::strQueryKDB(str, result, 10000);
 printf("\nCLIPS answer: %d %s\n",num,result.c_str());


}
